/*
### Written by Torsten Moeller
### sometime in March 1998
### let me know, if you find any errors (torsten@cs.sfu.ca)
###
### March 2003: conversion to a C++ class
###
### 1. it is now a C++ class with a bunch of (hopefully useful) methods
###    - size (returns the size of the filter)
###    - accuracy (an integer that tells the numeric asymthotic accuracy
###    - derivative (an integer that returns whether the filter is an
###      interpolation (0), first derivative (1), etc. filter
###    - continuity (an integer denoting the continuity class,
###                  -1 means not continuous)
### 2. you call it now as float filter(float offset)
*/

#ifndef _TMFILTERS_H_
#define _TMFILTERS_H_

#include "tmFilters_raw.h"

class tmFilter
{
 public:
  // the default filter will be C^{-1} continuous 1EF interpolation filter.
  tmFilter();
  // this sets the filter to be used
  tmFilter(int derivative, int continuity, int accuracy);
  virtual ~tmFilter() {};

  // this is the main thing of this class - it returns filter values
  float operator[](float x);
  float filter(float x); // an alernative interface

  // We can select which filter to use
  void setFunc(int derivative, int continuity, int accuracy);
  // specifies the open parameter that some (not all!) filters have
  void setA(float a);
  // We can output the filter - we just need to specify the sampling distance
  void print(float T);
  // what is the (non-zero) length of the filter?
  float getSize(void);

 private:
  // We can select which filter to use
  void pickFilter(int derivative, int continuity, int accuracy);

 private:
  int D; // derivative
  int C; // continuity
  int A; // accuracy
  float a; // open parameter
  float size; // length of non-zero part of the filter
  float (*m_filter)(float, float); // the actual filter, but it needs to include an additional parameter!

};

#endif
