/*
### Written by Torsten Moeller
### sometime in March 1998
### let me know, if you find any errors (moeller@cis.ohio-state.edu)
###
### March 2003 - modified to remove coeff part
*/

#pragma warning(disable:4244)
#pragma warning(disable:4305)

#define OVER_3 0.33333333
#define OVER_6 0.16666666
#define OVER_12 0.0833333333
#define OVER_2_3 0.6666666666

#include "tmFilters_raw.h"

float  wm4h, wm4g, wm4f, wm4e, wm4d, wm4c, wm4b, wm4a;
float  wm3h, wm3g, wm3f, wm3e, wm3d, wm3c, wm3b, wm3a;
float  wm2h, wm2g, wm2f, wm2e, wm2d, wm2c, wm2b, wm2a;
float  wm1h, wm1g, wm1f, wm1e, wm1d, wm1c, wm1b, wm1a;
float  wm0h, wm0g, wm0f, wm0e, wm0d, wm0c, wm0b, wm0a;
float  wp1h, wp1g, wp1f, wp1e, wp1d, wp1c, wp1b, wp1a;
float  wp2h, wp2g, wp2f, wp2e, wp2d, wp2c, wp2b, wp2a;
float  wp3h, wp3g, wp3f, wp3e, wp3d, wp3c, wp3b, wp3a;



/***************************************************************************/
/*                          Interpolation Filters                          */
/***************************************************************************/

float d0_cn_1ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm1b =  0; wm1a = 0.5;
  wm0b =  0; wm0a = 0.5;

  switch (i) {
  case 0: result = 0.5; break;
  case 1: result = 0.5; break;
  default: result = 0;
  }
  return result;
}

float d0_cn_2ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  switch (i) {
  case 0: result = t;   break;
  case 1: 1-t; break;
  default: result = 0;
  }
  return result;
}

float d0_cn_3ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm2c =  0.25; wm2b =  (2*a-0.25); wm2a = -(  a  );
  wm1c = -0.25; wm1b = -(6*a-1.25); wm1a =  (3*a  );
  wm0c = -0.25; wm0b =  (6*a-0.75); wm0a = -(3*a-1);
  wp1c =  0.25; wp1b = -(2*a+0.25); wp1a =  (  a  );

  switch (i) {
  case 0: result = (wm2c*t +wm2b)*t + wm2a; break;
  case 1: result = (wm1c*t +wm1b)*t + wm1a; break;
  case 2: result = (wm0c*t +wm0b)*t + wm0a; break;
  case 3: result = (wp1c*t +wp1b)*t + wp1a; break;
  default: result = 0;
  }
  return result;
}

float d0_cn_4ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm2d =  OVER_6; wm2c =    0; wm2b = -OVER_6; wm2a = 0;
  wm1d =    -0.5; wm1c =  0.5; wm1b =       1; wm1a = 0;
  wm0d =     0.5; wm0c = -1  ; wm0b =    -0.5; wm0a = 1;
  wp1d = -OVER_6; wp1c =  0.5; wp1b = -OVER_3; wp1a = 0;

  switch (i) {
  case 0: result = ((wm2d*t +wm2c)*t +wm2b)*t + wm2a; break;
  case 1: result = ((wm1d*t +wm1c)*t +wm1b)*t + wm1a; break;
  case 2: result = ((wm0d*t +wm0c)*t +wm0b)*t + wm0a; break;
  case 3: result = ((wp1d*t +wp1c)*t +wp1b)*t + wp1a; break;
  default: result = 0;
  }
  return result;
}

float d0_c0_1ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  switch (i) {
  case 0: result = result = t;   break;
  case 1: result = result = 1-t; break;
  default: result = 0;
  }
  return result;
}

float d0_c0_2ef(float a, float t) {

  return d0_c0_1ef(a, t);
}

float d0_c0_3ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  switch (i) {
  case 0: result = ( t -1)*t/4;    break;
  case 1: result = (-t +5)*t/4;    break;
  case 2: result = (-t -3)*t/4 +1; break;
  case 3: result = ( t -1)*t/4;    break;
  default: result = 0;
  }
  return result;
}

float d0_c0_4ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm2d =  OVER_6; wm2c =    0; wm2b = -OVER_6; wm2a = 0;
  wm1d =    -0.5; wm1c =  0.5; wm1b =       1; wm1a = 0;
  wm0d =     0.5; wm0c = -  1; wm0b =    -0.5; wm0a = 1;
  wp1d = -OVER_6; wp1c =  0.5; wp1b = -OVER_3; wp1a = 0;

  switch (i) {
  case 0: result = ((wm2d*t +wm2c)*t +wm2b)*t;    break;
  case 1: result = ((wm1d*t +wm1c)*t +wm1b)*t;    break;
  case 2: result = ((wm0d*t +wm0c)*t +wm0b)*t +1; break;
  case 3: result = ((wp1d*t +wp1c)*t +wp1b)*t;    break;
  default: result = 0;
  }
  return result;
}

float d0_c1_1ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm1d = -2; wm1c =  3;
  wm0d =  2; wm0c = -3;

  switch (i) {
  case 0: result = (wm1d*t +wm1c)*t*t;    break;
  case 1: result = (wm0d*t +wm0c)*t*t +1; break;
  default: result = 0;
  }
  return result;
}

float d0_c1_2ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm1a = 0.25;
  wm0a = 0.5;
  wp1a = 0.25;

  switch (i) {
  case 0: result = ( t   )*t/4;       break;
  case 1: result = (-t +2)*t/4 +wm1a; break;
  case 2: result = (-t   )*t/4 +wm0a; break;
  case 3: result = ( t -2)*t/4 +wp1a; break;
  default: result = 0;
  }
  return result;
}

float d0_c1_3ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm2d =  0.5; wm2c = -0.5; wm2b =    0; wm2a = 0;
  wm1d = -1.5; wm1c =    2; wm1b =  0.5; wm1a = 0;
  wm0d =  1.5; wm0c = -2.5; wm0b =    0; wm0a = 1;
  wp1d = -0.5; wp1c =    1; wp1b = -0.5; wp1a = 0;

  switch (i) {
  case 0: result =((wm2d*t +wm2c)*t +wm2b)*t;     break;
  case 1: result = ((wm1d*t +wm1c)*t +wm1b)*t;    break;
  case 2: result = ((wm0d*t +wm0c)*t +wm0b)*t +1; break;
  case 3: result = ((wp1d*t +wp1c)*t +wp1b)*t;    break;
  default: result = 0;
  }
  return result;
}

float d0_c1_4ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm3d = -(   a       ); wm3c =  (1.5*a- 1./24); wm3b =          0; wm3a =             0;
  wm2d =  ( 5*a+OVER_6); wm2c = -(7.5*a- 1./8 ); wm2b = -(OVER_12); wm2a =  (0.5*a-1./24);
  wm1d = -(10*a+   0.5); wm1c =  ( 15*a+ 5./12); wm1b =  (OVER_2_3); wm1a = -(  2*a-OVER_6 );
  wm0d =  (10*a+   0.5); wm0c = -( 15*a+13./12); wm0b =          0; wm0a =  (  3*a+0.75 );
  wp1d = -( 5*a+OVER_6); wp1c =  (7.5*a+ 5./8 ); wp1b = -(OVER_2_3); wp1a = -(  2*a-OVER_6 );
  wp2d =  (   a       ); wp2c = -(1.5*a+ 1./24); wp2b =  (OVER_12); wp2a =  (0.5*a-1./24);

  switch (i) {
  case 0: result = ((wm3d*t +wm3c)*t +wm3b)*t +wm3a; break;
  case 1: result = ((wm2d*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 2: result = ((wm1d*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 3: result = ((wm0d*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 4: result = ((wp1d*t +wp1c)*t +wp1b)*t +wp1a; break;
  case 5: result = ((wp2d*t +wp2c)*t +wp2b)*t +wp2a; break;
  default: result = 0;
  }
  return result;
}

float d0_c2_1ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm1f =  6; wm1e = -15; wm1d =  10; wm1c =  0; wm1b =  0; wm1a = 0;
  wm0f = -6; wm0e =  15; wm0d = -10; wm0c =  0; wm0b =  0; wm0a = 1;

  switch (i) {
  case 0: result = ((wm1f*t +wm1e)*t +wm1d)*t*t*t;    break;
  case 1: result = ((wm0f*t +wm0e)*t +wm0d)*t*t*t +1; break;
  default: result = 0;
  }
  return result;
}

float d0_c2_2ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm2d =  OVER_6; wm2c =    0; wm2b =    0; wm2a =   0;
  wm1d =    -0.5; wm1c =  0.5; wm1b =  0.5; wm1a = OVER_6;
  wm0d =     0.5; wm0c = -  1; wm0b =    0; wm0a = OVER_2_3;
  wp1d = -OVER_6; wp1c =  0.5; wp1b = -0.5; wp1a = OVER_6;

  switch (i) {
  case 0: result = ((wm2d*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 1: result = ((wm1d*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 2: result = ((wm0d*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 3: result = ((wp1d*t +wp1c)*t +wp1b)*t +wp1a; break;
  default: result = 0;
  }
  return result;
}

float d0_c2_3ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm2f = -1; wm2e =   2.5; wm2d = -1.5; wm2c =    0; wm2b =    0; wm2a = 0;
  wm1f =  3; wm1e = - 7.5; wm1d =  4.5; wm1c =  0.5; wm1b =  0.5; wm1a = 0;
  wm0f = -3; wm0e =   7.5; wm0d = -4.5; wm0c = -  1; wm0b =    0; wm0a = 1;
  wp1f =  1; wp1e = - 2.5; wp1d =  1.5; wp1c =  0.5; wp1b = -0.5; wp1a = 0;

  switch (i) {
  case 0: result = ((((wm2f*t +wm2e)*t +wm2d)*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 1: result = ((((wm1f*t +wm1e)*t +wm1d)*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 2: result = ((((wm0f*t +wm0e)*t +wm0d)*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 3: result = ((((wp1f*t +wp1e)*t +wp1d)*t +wp1c)*t +wp1b)*t +wp1a; break;
  default: result = 0;
  }
  return result;
}

float d0_c2_4ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm3d = - 1./36; wm3c =        0; wm3b =         0; wm3a =     0;
  wm2d =  11./36; wm2c = -OVER_12; wm2b =  -OVER_12; wm2a = -1./36;
  wm1d = - 7./9 ; wm1c =    5./6 ; wm1b =  OVER_2_3; wm1a =  1./9 ;
  wm0d =   7./9 ; wm0c =    -1.5 ; wm0b =         0; wm0a =  5./6 ;
  wp1d = -11./36; wp1c =    5./6 ; wp1b = -OVER_2_3; wp1a =  1./9 ;
  wp2d =   1./36; wp2c = -OVER_12; wp2b =   OVER_12; wp2a = -1./36;

  switch (i) {
  case 0: result = ((wm3d*t +wm3c)*t +wm3b)*t +wm3a; break;
  case 1: result = ((wm2d*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 2: result = ((wm1d*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 3: result = ((wm0d*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 4: result = ((wp1d*t +wp1c)*t +wp1b)*t +wp1a; break;
  case 5: result = ((wp2d*t +wp2c)*t +wp2b)*t +wp2a; break;
  default: result = 0;
  }
  return result;
}

float d0_c3_1ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm1h = -20; wm1g =  70; wm1f = -84; wm1e =  35; wm1d =   0; wm1c =  0; wm1b =  0; wm1a = 0;
  wm0h =  20; wm0g = -70; wm0f =  84; wm0e = -35; wm0d =   0; wm0c =  0; wm0b =  0; wm0a = 1;

  switch (i) {
  case 0: result = (((wm1h*t +wm1g)*t +wm1f)*t +wm1e)*t*t*t*t;    break;
  case 1: result = (((wm0h*t +wm0g)*t +wm0f)*t +wm0e)*t*t*t*t +1; break;
  default: result = 0;
  }
  return result;
}

float d0_c3_2ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm2f = -0.10; wm2e =  0.25; wm2d = 0; wm2c =    0; wm2b =    0; wm2a =    0;
  wm1f =  0.30; wm1e = -0.75; wm1d = 0; wm1c =  0.5; wm1b =  0.5; wm1a = 0.15;
  wm0f = -0.30; wm0e =  0.75; wm0d = 0; wm0c = -  1; wm0b =    0; wm0a = 0.70;
  wp1f =  0.10; wp1e = -0.25; wp1d = 0; wp1c =  0.5; wp1b = -0.5; wp1a = 0.15;

  switch (i) {
  case 0: result =    (wm2f*t +wm2e)*t*t*t*t;                            break;
  case 1: result = ((((wm1f*t +wm1e)*t +wm1d)*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 2: result = ((((wm0f*t +wm0e)*t +wm0d)*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 3: result = ((((wp1f*t +wp1e)*t +wp1d)*t +wp1c)*t +wp1b)*t +wp1a; break;
  default: result = 0;
  }
  return result;
}

float d0_c3_3ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm2h =  3; wm2g = -10.5; wm2f =  12.5; wm2e = - 5; wm2d = 0; wm2c =    0; wm2b =    0; wm2a = 0;
  wm1h = -9; wm1g =  31.5; wm1f = -37.5; wm1e =  15; wm1d = 0; wm1c =  0.5; wm1b =  0.5; wm1a = 0;
  wm0h =  9; wm0g = -31.5; wm0f =  37.5; wm0e = -15; wm0d = 0; wm0c = -  1; wm0b =    0; wm0a = 1;
  wp1h = -3; wp1g =  10.5; wp1f = -12.5; wp1e =   5; wp1d = 0; wp1c =  0.5; wp1b = -0.5; wp1a = 0;

  switch (i) {
  case 0: result = ((((((wm2h*t +wm2g)*t +wm2f)*t +wm2e)*t +wm2d)*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 1: result = ((((((wm1h*t +wm1g)*t +wm1f)*t +wm1e)*t +wm1d)*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 2: result = ((((((wm0h*t +wm0g)*t +wm0f)*t +wm0e)*t +wm0d)*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 3: result = ((((((wp1h*t +wp1g)*t +wp1f)*t +wp1e)*t +wp1d)*t +wp1c)*t +wp1b)*t +wp1a; break;
  default: result = 0;
  }
  return result;
}

float d0_c3_4ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm3f =   1./30; wm3e = - 1./16; wm3d =        0; wm3c =        0; wm3b =         0; wm3a = 0;
  wm2f = -OVER_6; wm2e =  17./48; wm2d =  OVER_12; wm2c =   -1./24; wm2b =  -OVER_12; wm2a = - 7./240;
  wm1f =  OVER_3; wm1e = -19./24; wm1d = -OVER_6 ; wm1c = OVER_2_3; wm1b =  OVER_2_3; wm1a =   7./60;
  wm0f = -OVER_3; wm0e =   7./8 ; wm0d =        0; wm0c =    -1.25; wm0b =         0; wm0a =  33./40;
  wp1f =  OVER_6; wp1e = -23./48; wp1d =  OVER_6 ; wp1c = OVER_2_3; wp1b = -OVER_2_3; wp1a =   7./60;
  wp2f =  -1./30; wp2e =   5./48; wp2d = -OVER_12; wp2c =   -1./24; wp2b =   OVER_12; wp2a = - 7./240;

  switch (i) {
  case 0: result = ((((wm3f*t +wm3e)*t +wm3d)*t +wm3c)*t +wm3b)*t +wm3a; break;
  case 1: result = ((((wm2f*t +wm2e)*t +wm2d)*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 2: result = ((((wm1f*t +wm1e)*t +wm1d)*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 3: result = ((((wm0f*t +wm0e)*t +wm0d)*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 4: result = ((((wp1f*t +wp1e)*t +wp1d)*t +wp1c)*t +wp1b)*t +wp1a; break;
  case 5: result = ((((wp2f*t +wp2e)*t +wp2d)*t +wp2c)*t +wp2b)*t +wp2a; break;
  default: result = 0;
  }
  return result;
}

/***************************************************************************/
/*                        First Derivative Filters                         */
/***************************************************************************/


float d1_cn_1ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  switch (i) {
  case 0: result =  1; break;
  case 1: result = -1; break;
  default: result = 0;
  }
  return result;
}

float d1_cn_2ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm2b =  0.5; wm2a =  (  a    );
  wm1b = -0.5; wm1a = -(3*a-0.5);
  wm0b = -0.5; wm0a =  (3*a    );
  wp1b =  0.5; wp1a = -(  a+0.5);

  switch (i) {
  case 0: result = wm2b*t +wm2a; break;
  case 1: result = wm1b*t +wm1a; break;
  case 2: result = wm0b*t +wm0a; break;
  case 3: result = wp1b*t +wp1a; break;
  default: result = 0;
  }
  return result;
}

float d1_cn_3ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm2c =  0.5; wm2b =  0; wm2a = -OVER_6;
  wm1c = -1.5; wm1b =  1; wm1a =       1;
  wm0c =  1.5; wm0b = -2; wm0a =    -0.5;
  wp1c = -0.5; wp1b =  1; wp1a = -OVER_3;

  switch (i) {
  case 0: result = (wm2c*t +wm2b)*t +wm2a; break;
  case 1: result = (wm1c*t +wm1b)*t +wm1a; break;
  case 2: result = (wm0c*t +wm0b)*t +wm0a; break;
  case 3: result = (wp1c*t +wp1b)*t +wp1a; break;
  default: result = 0;
  }
  return result;
}

float d1_cn_4ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm3d =  OVER_12; wm3c =  (   a     ); wm3b = -(   a+ OVER_6); wm3a =  0;
  wm2d =    -0.25; wm2c = -( 5*a-0.25); wm2b =  ( 5*a+   0.75); wm2a = -OVER_12;
  wm1d =   OVER_6; wm1c =  (10*a- 0.5); wm1b = -(10*a+ OVER_3); wm1a =  OVER_2_3;
  wm0d =   OVER_6; wm0c = -(10*a     ); wm0b =  (10*a-   5./6); wm0a = 0;
  wp1d =    -0.25; wp1c =  ( 5*a+ 0.5); wp1b = -( 5*a-    0.5); wp1a = -OVER_2_3;
  wp2d =  OVER_12; wp2c = -(   a+0.25); wp2b =  (   a+OVER_12); wp2a =  OVER_12;

  switch (i) {
  case 0: result =((wm3d*t +wm3c)*t +wm3b)*t +wm3a; break;
  case 1: result = ((wm2d*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 2: result = ((wm1d*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 3: result = ((wm0d*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 4: result = ((wp1d*t +wp1c)*t +wp1b)*t +wp1a; break;
  case 5: result = ((wp2d*t +wp2c)*t +wp2b)*t +wp2a; break;
  default: result = 0;
  }
  return result;
}

float d1_c0_1ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  switch (i) {
  case 0: result = t/2;     break;
  case 1: result = (1-t)/2; break;
  case 2: result = -t/2;    break;
  case 3: result = (t-1)/2; break;
  default: result = 0;
  }
  return result;
}

float d1_c0_2ef(float a, float t) {

  return d1_c0_1ef(a, t);
}

float d1_c0_3ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm3c =  (   a    ); wm3b = -(   a+OVER_12); wm3a = 0;
  wm2c = -( 5*a-0.5); wm2b =  ( 5*a+   0.25); wm2a = -OVER_12;
  wm1c =  (10*a-1.5); wm1b = -(10*a-   5./6); wm1a =  OVER_2_3;
  wm0c = -(10*a-1.5); wm0b =  (10*a-  13./6); wm0a = 0;
  wp1c =  ( 5*a-0.5); wp1b = -( 5*a-   1.25); wp1a = -OVER_2_3;
  wp2c = -(   a    ); wp2b =  (   a-OVER_12); wp2a =  OVER_12;

  switch (i) {
  case 0: result = (wm3c*t +wm3b)*t +wm3a; break;
  case 1: result = (wm2c*t +wm2b)*t +wm2a; break;
  case 2: result = (wm1c*t +wm1b)*t +wm1a; break;
  case 3: result = (wm0c*t +wm0b)*t +wm0a; break;
  case 4: result = (wp1c*t +wp1b)*t +wp1a; break;
  case 5: result = (wp2c*t +wp2b)*t +wp2a; break;
  default: result = 0;
  }
  return result;
}

float d1_c0_4ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm3d = OVER_12; wm3c =  (   a     ); wm3b = -(   a+ OVER_6); wm3a =       0;
  wm2d =   -0.25; wm2c = -( 5*a-0.25); wm2b =  ( 5*a+   0.75); wm2a = -OVER_12;
  wm1d =  OVER_6; wm1c =  (10*a- 0.5); wm1b = -(10*a+ OVER_3); wm1a =  OVER_2_3;
  wm0d =  OVER_6; wm0c = -(10*a     ); wm0b =  (10*a-   5./6); wm0a =       0;
  wp1d =   -0.25; wp1c =  ( 5*a+ 0.5); wp1b = -( 5*a-    0.5); wp1a = -OVER_2_3;
  wp2d = OVER_12; wp2c = -(   a+0.25); wp2b =  (   a+OVER_12); wp2a =  OVER_12;

  switch (i) {
  case 0: result = ((wm3d*t +wm3c)*t +wm3b)*t +wm3a; break;
  case 1: result = ((wm2d*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 2: result = ((wm1d*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 3: result = ((wm0d*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 4: result = ((wp1d*t +wp1c)*t +wp1b)*t +wp1a; break;
  case 5: result = ((wp2d*t +wp2c)*t +wp2b)*t +wp2a; break;
  default: result = 0;
  }
  return result;
}

float d1_c1_1ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm2c =  0.5; wm2b =  0; wm2a =    0;
  wm1c = -1.5; wm1b =  1; wm1a =  0.5;
  wm0c =  1.5; wm0b = -2; wm0a =    0;
  wp1c = -0.5; wp1b =  1; wp1a = -0.5;

  switch (i) {
  case 0: result = (wm2c*t +wm2b)*t +wm2a; break;
  case 1: result = (wm1c*t +wm1b)*t +wm1a; break;
  case 2: result = (wm0c*t +wm0b)*t +wm0a; break;
  case 3: result = (wp1c*t +wp1b)*t +wp1a; break;
  default: result = 0;
  }
  return result;
}

float d1_c1_2ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm2c =  0.5; wm2b =  0; wm2a =    0;
  wm1c = -1.5; wm1b =  1; wm1a =  0.5;
  wm0c =  1.5; wm0b = -2; wm0a =    0;
  wp1c = -0.5; wp1b =  1; wp1a = -0.5;

  switch (i) {
  case 0: result = (wm2c*t +wm2b)*t +wm2a; break;
  case 1: result = (wm1c*t +wm1b)*t +wm1a; break;
  case 2: result = (wm0c*t +wm0b)*t +wm0a; break;
  case 3: result = (wp1c*t +wp1b)*t +wp1a; break;
  default: result = 0;
  }
  return result;
}

float d1_c1_3ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm3c = -OVER_12; wm3b =       0; wm3a =  0;
  wm2c =   11./12; wm2b = -OVER_6; wm2a = -OVER_12;
  wm1c =    -7./3; wm1b =    5./3; wm1a =  OVER_2_3;
  wm0c =     7./3; wm0b =      -3; wm0a =  0;
  wp1c =  -11./12; wp1b =    5./3; wp1a = -OVER_2_3;
  wp2c =  OVER_12; wp2b = -OVER_6; wp2a =  OVER_12;

  switch (i) {
  case 0: result = (wm3c*t +wm3b)*t +wm3a; break;
  case 1: result = (wm2c*t +wm2b)*t +wm2a; break;
  case 2: result = (wm1c*t +wm1b)*t +wm1a; break;
  case 3: result = (wm0c*t +wm0b)*t +wm0a; break;
  case 4: result = (wp1c*t +wp1b)*t +wp1a; break;
  case 5: result = (wp2c*t +wp2b)*t +wp2a; break;
  default: result = 0;
  }
  return result;
}

float d1_c1_4ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm3d =  OVER_12; wm3c =  -OVER_6; wm3b =        0; wm3a =  0;
  wm2d =    -0.25; wm2c =   13./12; wm2b = -OVER_12; wm2a = -OVER_12;
  wm1d =   OVER_6; wm1c =   -13./6; wm1b =     4./3; wm1a =  OVER_2_3 ;
  wm0d =   OVER_6; wm0c =     5./3; wm0b =     -2.5; wm0a =  0;
  wp1d =    -0.25; wp1c =  -OVER_3; wp1b =     4./3; wp1a = -OVER_2_3;
  wp2d =  OVER_12; wp2c = -OVER_12; wp2b = -OVER_12; wp2a =  OVER_12;

  switch (i) {
  case 0: result = ((wm3d*t +wm3c)*t +wm3b)*t +wm3a; break;
  case 1: result = ((wm2d*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 2: result = ((wm1d*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 3: result = ((wm0d*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 4: result = ((wp1d*t +wp1c)*t +wp1b)*t +wp1a; break;
  case 5: result = ((wp2d*t +wp2c)*t +wp2b)*t +wp2a; break;
  default: result = 0;
  }
  return result;
}

float d1_c2_1ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm2e = -0.5; wm2d =  1; wm2c = 0; wm2b =  0; wm2a =    0;
  wm1e =  1.5; wm1d = -3; wm1c = 0; wm1b =  1; wm1a =  0.5;
  wm0e = -1.5; wm0d =  3; wm0c = 0; wm0b = -2; wm0a =    0;
  wp1e =  0.5; wp1d = -1; wp1c = 0; wp1b =  1; wp1a = -0.5;

  switch (i) {
  case 0: result = ((wm2e*t +wm2d)*t*t +wm2b)*t +wm2a; break;
  case 1: result = ((wm1e*t +wm1d)*t*t +wm1b)*t +wm1a; break;
  case 2: result = ((wm0e*t +wm0d)*t*t +wm0b)*t +wm0a; break;
  case 3: result = ((wp1e*t +wp1d)*t*t +wp1b)*t +wp1a; break;
  default: result = 0;
  }
  return result;
}

float d1_c2_2ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm2e = -0.5; wm2d =  1; wm2c = 0; wm2b =  0; wm2a =  0;
  wm1e =  1.5; wm1d = -3; wm1c = 0; wm1b =  1; wm1a =  0.5;
  wm0e = -1.5; wm0d =  3; wm0c = 0; wm0b = -2; wm0a =  0;
  wp1e =  0.5; wp1d = -1; wp1c = 0; wp1b =  1; wp1a = -0.5;

  switch (i) {
  case 0: result = (((wm2e*t +wm2d)*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 1: result = (((wm1e*t +wm1d)*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 2: result = (((wm0e*t +wm0d)*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 3: result = (((wp1e*t +wp1d)*t +wp1c)*t +wp1b)*t +wp1a; break;
  default: result = 0;
  }
  return result;
}

float d1_c2_3ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm3e =  OVER_6; wm3d =   -0.25; wm3c =     0; wm3b =        0; wm3a =  0;
  wm2e =   -5./6; wm2d =  17./12; wm2c =  0.25; wm2b = -OVER_12; wm2a = -OVER_12;
  wm1e =    5./3; wm1d =  -19./6; wm1c =  -0.5; wm1b =     4./3; wm1a =  OVER_2_3;
  wm0e =   -5./3; wm0d =     3.5; wm0c =     0; wm0b =     -2.5; wm0a =  0;
  wp1e =    5./6; wp1d = -23./12; wp1c =   0.5; wp1b =     4./3; wp1a = -OVER_2_3;
  wp2e = -OVER_6; wp2d =   5./12; wp2c = -0.25; wp2b = -OVER_12; wp2a =  OVER_12;

  switch (i) {
  case 0: result = (((wm3e*t +wm3d)*t +wm3c)*t +wm3b)*t +wm3a; break;
  case 1: result = (((wm2e*t +wm2d)*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 2: result = (((wm1e*t +wm1d)*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 3: result = (((wm0e*t +wm0d)*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 4: result = (((wp1e*t +wp1d)*t +wp1c)*t +wp1b)*t +wp1a; break;
  case 5: result = (((wp2e*t +wp2d)*t +wp2c)*t +wp2b)*t +wp2a; break;
  default: result = 0;
  }
  return result;
}

float d1_c2_4ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm3e =  OVER_6; wm3d =   -0.25; wm3c =     0; wm3b =        0; wm3a =  0;
  wm2e =   -5./6; wm2d =  17./12; wm2c =  0.25; wm2b = -OVER_12; wm2a = -OVER_12;
  wm1e =    5./3; wm1d =  -19./6; wm1c =  -0.5; wm1b =     4./3; wm1a =  OVER_2_3;
  wm0e =   -5./3; wm0d =     3.5; wm0c =     0; wm0b =     -2.5; wm0a =  0;
  wp1e =    5./6; wp1d = -23./12; wp1c =   0.5; wp1b =     4./3; wp1a = -OVER_2_3;
  wp2e = -OVER_6; wp2d =   5./12; wp2c = -0.25; wp2b = -OVER_12; wp2a =  OVER_12;

  switch (i) {
  case 0: result = (((wm3e*t +wm3d)*t +wm3c)*t +wm3b)*t +wm3a; break;
  case 1: result = (((wm2e*t +wm2d)*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 2: result = (((wm1e*t +wm1d)*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 3: result = (((wm0e*t +wm0d)*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 4: result = (((wp1e*t +wp1d)*t +wp1c)*t +wp1b)*t +wp1a; break;
  case 5: result = (((wp2e*t +wp2d)*t +wp2c)*t +wp2b)*t +wp2a; break;
  default: result = 0;
  }
  return result;
}

float d1_c3_1ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm2f = -0.75; wm2e =  1.25; wm2d =    0; wm2c = 0; wm2b =     0; wm2a =  0;
  wm1f =  0.75; wm1e =     0; wm1d = -2.5; wm1c = 0; wm1b =  1.25; wm1a =  0.5;
  wm0f =  0.75; wm0e = -3.75; wm0d =    5; wm0c = 0; wm0b =  -2.5; wm0a =  0;
  wp1f = -0.75; wp1e =  2.5 ; wp1d = -2.5; wp1c = 0; wp1b =  1.25; wp1a = -0.5;

  switch (i) {
  case 0: result = ((((wm2f*t +wm2e)*t +wm2d)*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 1: result = ((((wm1f*t +wm1e)*t +wm1d)*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 2: result = ((((wm0f*t +wm0e)*t +wm0d)*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 3: result = ((((wp1f*t +wp1e)*t +wp1d)*t +wp1c)*t +wp1b)*t +wp1a; break;
  default: result = 0;
  }
  return result;
}

float d1_c3_2ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm2g =  1; wm2f = -3; wm2e =  2.5; wm2b =  0; wm2a =  0;
  wm1g = -3; wm1f =  9; wm1e = -7.5; wm1b =  1; wm1a =  0.5;
  wm0g =  3; wm0f = -9; wm0e =  7.5; wm0b = -2; wm0a =  0;
  wp1g = -1; wp1f =  3; wp1e = -2.5; wp1b =  1; wp1a = -0.5;

  switch (i) {
  case 0: result = (((wm2g*t +wm2f)*t +wm2e)*t*t*t +wm2b)*t +wm2a; break;
  case 1: result = (((wm1g*t +wm1f)*t +wm1e)*t*t*t +wm1b)*t +wm1a; break;
  case 2: result = (((wm0g*t +wm0f)*t +wm0e)*t*t*t +wm0b)*t +wm0a; break;
  case 3: result = (((wp1g*t +wp1f)*t +wp1e)*t*t*t +wp1b)*t +wp1a; break;
  default: result = 0;
  }
  return result;
}

float d1_c3_3ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm3f =  3./16; wm3e =   -13./48; wm3d =       0; wm3c =     0; wm3b =       0; wm3a =  0;
  wm2f = -9./16; wm2e =     5./12; wm2d =  19./24; wm2c =  0.25; wm2b = - 7./48; wm2a = -OVER_12;
  wm1f =  3./8 ; wm1e =    25./24; wm1d = -19./6 ; wm1c =  -0.5; wm1b =  19./12; wm1a =  OVER_2_3;
  wm0f =  3./8 ; wm0e =   -35./12; wm0d =  19./4 ; wm0c =     0; wm0b = -23./8 ; wm0a =  0;
  wp1f = -9./16; wp1e =   115./48; wp1d = -19./6 ; wp1c =   0.5; wp1b =  19./12; wp1a = -OVER_2_3;
  wp2f =  3./16; wp2e = -OVER_2_3; wp2d =  19./24; wp2c = -0.25; wp2b = - 7./48; wp2a =  OVER_12;

  switch (i) {
  case 0: result = ((((wm3f*t +wm3e)*t +wm3d)*t +wm3c)*t +wm3b)*t +wm3a; break;
  case 1: result = ((((wm2f*t +wm2e)*t +wm2d)*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 2: result = ((((wm1f*t +wm1e)*t +wm1d)*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 3: result = ((((wm0f*t +wm0e)*t +wm0d)*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 4: result = ((((wp1f*t +wp1e)*t +wp1d)*t +wp1c)*t +wp1b)*t +wp1a; break;
  case 5: result = ((((wp2f*t +wp2e)*t +wp2d)*t +wp2c)*t +wp2b)*t +wp2a; break;
  default: result = 0;
  }
  return result;
}

float d1_c3_4ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm3g = -0.25; wm3f =  0.75; wm3e = - 7./12; wm3d =         0; wm3c =     0; wm3b =        0; wm3a =  0;
  wm2g =  1.25; wm2f = -3.75; wm2e =  35./12; wm2d =    OVER_6; wm2c =  0.25; wm2b = -OVER_12; wm2a = -OVER_12;
  wm1g =  -2.5; wm1f =   7.5; wm1e = -35./6 ; wm1d = -OVER_2_3; wm1c =  -0.5; wm1b =     4./3; wm1a =  OVER_2_3;
  wm0g =   2.5; wm0f =  -7.5; wm0e =  35./6 ; wm0d =         1; wm0c =     0; wm0b =    -5./2; wm0a =  0;
  wp1g = -1.25; wp1f =  3.75; wp1e = -35./12; wp1d = -OVER_2_3; wp1c =   0.5; wp1b =     4./3; wp1a = -OVER_2_3;
  wp2g =  0.25; wp2f = -0.75; wp2e =   7./12; wp2d =    OVER_6; wp2c = -0.25; wp2b = -OVER_12; wp2a =  OVER_12;

  switch (i) {
  case 0: result = (((((wm3g*t +wm3f)*t +wm3e)*t +wm3d)*t +wm3c)*t +wm3b)*t +wm3a; break;
  case 1: result = (((((wm2g*t +wm2f)*t +wm2e)*t +wm2d)*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 2: result = (((((wm1g*t +wm1f)*t +wm1e)*t +wm1d)*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 3: result = (((((wm0g*t +wm0f)*t +wm0e)*t +wm0d)*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 4: result = (((((wp1g*t +wp1f)*t +wp1e)*t +wp1d)*t +wp1c)*t +wp1b)*t +wp1a; break;
  case 5: result = (((((wp2g*t +wp2f)*t +wp2e)*t +wp2d)*t +wp2c)*t +wp2b)*t +wp2a; break;
  default: result = 0;
  }
  return result;
}


/***************************************************************************/
/*                       Second Derivative Filters                         */
/***************************************************************************/


float d2_cn_1ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  switch (i) {
  case 0: result =  0.5; break;
  case 1: result = -0.5; break;
  case 2: result = -0.5; break;
  case 3: result =  0.5; break;
  default: result = 0;
  }
  return result;
}

float d2_cn_2ef(float a, float t) {

  return d2_c0_2ef(a, t);
}

float d2_cn_3ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm3c =  0.25; wm3b = -( 2*a+ OVER_3); wm3a =     a;
  wm2c = -0.75; wm2b =  (10*a+  13./6); wm2a = - 5*a - OVER_12;
  wm1c =  0.5 ; wm1b = -(20*a+  13./3); wm1a =  10*a +    4./3;
  wm0c =  0.5 ; wm0b =  (20*a+  10./3); wm0a = -10*a -     2.5;
  wp1c = -0.75; wp1b = -(10*a+   2./3); wp1a =   5*a +    4./3;
  wp2c =  0.25; wp2b =  ( 2*a- OVER_6); wp2a = -   a - OVER_12;

  switch (i) {
  case 0: result = (wm3c*t +wm3b)*t +wm3a; break;
  case 1: result = (wm2c*t +wm2b)*t +wm2a; break;
  case 2: result = (wm1c*t +wm1b)*t +wm1a; break;
  case 3: result = (wm0c*t +wm0b)*t +wm0a; break;
  case 4: result = (wp1c*t +wp1b)*t +wp1a; break;
  case 5: result = (wp2c*t +wp2b)*t +wp2a; break;
  default: result = 0;
  }
  return result;
}

float d2_cn_4ef(float a, float t) {

  return d2_c0_4ef(a, t);
}

float d2_c0_1ef(float a, float t) {

  return d2_c0_2ef(a, t);
}

float d2_c0_2ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  switch (i) {
  case 0: result =    t; break;
  case 1: result = -3*t + 1; break;
  case 2: result =  3*t - 2; break;
  case 3: result = -  t + 1; break;
  default: result = 0;
  }
  return result;
}

float d2_c0_3ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm3c =  0.25; wm3b = -OVER_3; wm3a =         0;
  wm2c = -0.75; wm2b =  13./6 ; wm2a = - OVER_12;
  wm1c =  0.5 ; wm1b = -13./3 ; wm1a =      4./3;
  wm0c =  0.5 ; wm0b =  10./3 ; wm0a = -     2.5;
  wp1c = -0.75; wp1b = -2./3  ; wp1a =      4./3;
  wp2c =  0.25; wp2b = -OVER_6; wp2a = - OVER_12;

  switch (i) {
  case 0: result = (wm3c*t +wm3b)*t +wm3a; break;
  case 1: result = (wm2c*t +wm2b)*t +wm2a; break;
  case 2: result = (wm1c*t +wm1b)*t +wm1a; break;
  case 3: result = (wm0c*t +wm0b)*t +wm0a; break;
  case 4: result = (wp1c*t +wp1b)*t +wp1a; break;
  case 5: result = (wp2c*t +wp2b)*t +wp2a; break;
  default: result = 0;
  }
  return result;
}

float d2_c0_4ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm3d =  1./6; wm3c =  0  ; wm3b = -0.25; wm3a =     0;
  wm2d = -5./6; wm2c =  0.5; wm2b =  1.75; wm2a = -1./12;
  wm1d =  5./3; wm1c = -2  ; wm1b = -3.5 ; wm1a =  4./3;
  wm0d = -5./3; wm0c =  3  ; wm0b =  2.5 ; wm0a = -2.5;
  wp1d =  5./6; wp1c = -2  ; wp1b = -0.25; wp1a =  4./3;
  wp2d = -1./6; wp2c =  0.5; wp2b = -0.25; wp2a = -1./12;

  switch (i) {
  case 0: result = ((wm3d*t +wm3c)*t +wm3b)*t +wm3a; break;
  case 1: result = ((wm2d*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 2: result = ((wm1d*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 3: result = ((wm0d*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 4: result = ((wp1d*t +wp1c)*t +wp1b)*t +wp1a; break;
  case 5: result = ((wp2d*t +wp2c)*t +wp2b)*t +wp2a; break;
  default: result = 0;
  }
  return result;
}

float d2_c1_1ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm2d = -2; wm2c =  3; wm2b = 0; wm2a =  0;
  wm1d =  6; wm1c = -9; wm1b = 0; wm1a =  1;
  wm0d = -6; wm0c =  9; wm0b = 0; wm0a = -2;
  wp1d =  2; wp1c = -3; wp1b = 0; wp1a =  1;

  switch (i) {
  case 0: result = ((wm2d*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 1: result = ((wm1d*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 2: result = ((wm0d*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 3: result = ((wp1d*t +wp1c)*t +wp1b)*t +wp1a; break;
  default: result = 0;
  }
  return result;
}

float d2_c1_2ef(float a, float t) {

  return d2_c1_3ef(a, t);
}

float d2_c1_3ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm3d =   2./3; wm3c = - 0.75; wm3b =  0  ; wm3a =     0;
  wm2d = -10./3; wm2c =   4.25; wm2b =  0.5; wm2a = -1./12;
  wm1d =  20./3; wm1c = - 9.5 ; wm1b = -1  ; wm1a =  4./3;
  wm0d = -20./3; wm0c =  10.5 ; wm0b =  0  ; wm0a = -2.5;
  wp1d =  10./3; wp1c = - 5.75; wp1b =  1  ; wp1a =  4./3;
  wp2d = - 2./3; wp2c =   1.25; wp2b = -0.5; wp2a = -1./12;

  switch (i) {
  case 0: result = ((wm3d*t +wm3c)*t +wm3b)*t +wm3a; break;
  case 1: result = ((wm2d*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 2: result = ((wm1d*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 3: result = ((wm0d*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 4: result = ((wp1d*t +wp1c)*t +wp1b)*t +wp1a; break;
  case 5: result = ((wp2d*t +wp2c)*t +wp2b)*t +wp2a; break;
  default: result = 0;
  }
  return result;
}

float d2_c1_4ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm4d = - 53./360; wm4c =   19./120; wm4b =      0; wm4a =   0;
  wm3d =  431./360; wm3c = - 37./30 ; wm3b = -1./8 ; wm3a =   1./90;
  wm2d = -157./40 ; wm2c =  183./40 ; wm2b =  1    ; wm2a = - 3./20;
  wm1d =  491./72 ; wm1c = -113./12 ; wm1b = -13./8; wm1a =   3./2 ;
  wm0d = -491./72 ; wm0c =  265./24 ; wm0b =  0    ; wm0a = -49./18;
  wp1d =  157./40 ; wp1c = - 36./5  ; wp1b =  13./8; wp1a =   3./2 ;
  wp2d = -431./360; wp2c =  283./120; wp2b = -1    ; wp2a = - 3./20;
  wp3d =   53./360; wp3c = - 17./60 ; wp3b =  1./8 ; wp3a =   1./90;

  switch (i) {
  case 0: result = ((wm4d*t +wm4c)*t +wm4b)*t +wm4a; break;
  case 1: result = ((wm3d*t +wm3c)*t +wm3b)*t +wm3a; break;
  case 2: result = ((wm2d*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 3: result = ((wm1d*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 4: result = ((wm0d*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 5: result = ((wp1d*t +wp1c)*t +wp1b)*t +wp1a; break;
  case 6: result = ((wp2d*t +wp2c)*t +wp2b)*t +wp2a; break;
  case 7: result = ((wp3d*t +wp3c)*t +wp3b)*t +wp3a; break;
  default: result = 0;
  }
  return result;
}

float d2_c2_1ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm2f =   6; wm2e = -15; wm2d =  10; wm2c = 0; wm2b = 0; wm2a =  0;
  wm1f = -18; wm1e =  45; wm1d = -30; wm1c = 0; wm1b = 0; wm1a =  1;
  wm0f =  18; wm0e = -45; wm0d =  30; wm0c = 0; wm0b = 0; wm0a = -2;
  wp1f = - 6; wp1e =  15; wp1d = -10; wp1c = 0; wp1b = 0; wp1a =  1;

  switch (i) {
  case 0: result = ((((wm2f*t +wm2e)*t +wm2d)*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 1: result = ((((wm1f*t +wm1e)*t +wm1d)*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 2: result = ((((wm0f*t +wm0e)*t +wm0d)*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 3: result = ((((wp1f*t +wp1e)*t +wp1d)*t +wp1c)*t +wp1b)*t +wp1a; break;
  default: result = 0;
  }
  return result;
}

float d2_c2_2ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm3d =  1./6; wm3c =  0  ; wm3b =  0  ; wm3a =     0;
  wm2d = -5./6; wm2c =  0.5; wm2b =  0.5; wm2a =  1./6;
  wm1d =  5./3; wm1c = -2  ; wm1b = -1  ; wm1a =  1./3;
  wm0d = -5./3; wm0c =  3  ; wm0b =  0  ; wm0a = -1   ;
  wp1d =  5./6; wp1c = -2  ; wp1b =  1  ; wp1a =  1./3;
  wp2d = -1./6; wp2c =  0.5; wp2b = -0.5; wp2a =  1./6;

  switch (i) {
  case 0: result = ((wm3d*t +wm3c)*t +wm3b)*t +wm3a; break;
  case 1: result = ((wm2d*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 2: result = ((wm1d*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 3: result = ((wm0d*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 4: result = ((wp1d*t +wp1c)*t +wp1b)*t +wp1a; break;
  case 5: result = ((wp2d*t +wp2c)*t +wp2b)*t +wp2a; break;
  default: result = 0;
  }
  return result;
}

float d2_c2_3ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm3f = -1.5; wm3e =   3.75; wm3d = - 7./3; wm3c =    0; wm3b =  0  ; wm3a =  0;
  wm2f =  7.5; wm2e = -18.75; wm2d =  35./3; wm2c =  0.5; wm2b =  0.5; wm2a = -1./12;
  wm1f = -15 ; wm1e =  37.5 ; wm1d = -70./3; wm1c = -2  ; wm1b = -1  ; wm1a =  4./3;
  wm0f =  15 ; wm0e = -37.5 ; wm0d =  70./3; wm0c =  3  ; wm0b =  0  ; wm0a = -2.5;
  wp1f = -7.5; wp1e =  18.75; wp1d = -35./3; wp1c = -2  ; wp1b =  1  ; wp1a =  4./3;
  wp2f =  1.5; wp2e = - 3.75; wp2d =   7./3; wp2c =  0.5; wp2b = -0.5; wp2a = -1./12;

  switch (i) {
  case 0: result = ((((wm3f*t +wm3e)*t +wm3d)*t +wm3c)*t +wm3b)*t +wm3a; break;
  case 1: result = ((((wm2f*t +wm2e)*t +wm2d)*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 2: result = ((((wm1f*t +wm1e)*t +wm1d)*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 3: result = ((((wm0f*t +wm0e)*t +wm0d)*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 4: result = ((((wp1f*t +wp1e)*t +wp1d)*t +wp1c)*t +wp1b)*t +wp1a; break;
  case 5: result = ((((wp2f*t +wp2e)*t +wp2d)*t +wp2c)*t +wp2b)*t +wp2a; break;
  default: result = 0;
  }
  return result;
}

float d2_c2_4ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm4d = - 1./24; wm4c =      0; wm4b =      0; wm4a =   0;
  wm3d =  11./24; wm3c = - 1./8; wm3b = -1./8 ; wm3a = - 1./24;
  wm2d = -41./24; wm2c =   5./4; wm2b =  1    ; wm2a =   1./6 ;
  wm1d =  25./8 ; wm1c = -31./8; wm1b = -13./8; wm1a =  17./24;
  wm0d = -25./8 ; wm0c =  11./2; wm0b =  0    ; wm0a = - 5./3 ;
  wp1d =  41./24; wp1c = -31./8; wp1b =  13./8; wp1a =  17./24;
  wp2d = -11./24; wp2c =   5./4; wp2b = -1    ; wp2a =   1./6 ;
  wp3d =   1./24; wp3c = - 1./8; wp3b =  1./8 ; wp3a = - 1./24;

  switch (i) {
  case 0: result = ((wm4d*t +wm4c)*t +wm4b)*t +wm4a; break;
  case 1: result = ((wm3d*t +wm3c)*t +wm3b)*t +wm3a; break;
  case 2: result = ((wm2d*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 3: result = ((wm1d*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 4: result = ((wm0d*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 5: result = ((wp1d*t +wp1c)*t +wp1b)*t +wp1a; break;
  case 6: result = ((wp2d*t +wp2c)*t +wp2b)*t +wp2a; break;
  case 7: result = ((wp3d*t +wp3c)*t +wp3b)*t +wp3a; break;
  default: result = 0;
  }
  return result;
}

float d2_c3_1ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm2h = -20; wm2g =   70; wm2f = - 84; wm2e =   35; wm2d = 0; wm2c = 0; wm2b = 0; wm2a =  0;
  wm1h =  60; wm1g = -210; wm1f =  252; wm1e = -105; wm1d = 0; wm1c = 0; wm1b = 0; wm1a =  1;
  wm0h = -60; wm0g =  210; wm0f = -252; wm0e =  105; wm0d = 0; wm0c = 0; wm0b = 0; wm0a = -2;
  wp1h =  20; wp1g = - 70; wp1f =   84; wp1e = - 35; wp1d = 0; wp1c = 0; wp1b = 0; wp1a =  1;

  switch (i) {
  case 0: result = ((((((wm2h*t +wm2g)*t +wm2f)*t +wm2e)*t +wm2d)*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 1: result = ((((((wm1h*t +wm1g)*t +wm1f)*t +wm1e)*t +wm1d)*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 2: result = ((((((wm0h*t +wm0g)*t +wm0f)*t +wm0e)*t +wm0d)*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 3: result = ((((((wp1h*t +wp1g)*t +wp1f)*t +wp1e)*t +wp1d)*t +wp1c)*t +wp1b)*t +wp1a; break;
  default: result = 0;
  }
  return result;
}

float d2_c3_2ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm3f = -0.1; wm3e =  0.25; wm3d = 0; wm3c =    0; wm3b =  0  ; wm3a =  0;
  wm2f =  0.5; wm2e = -1.25; wm2d = 0; wm2c =  0.5; wm2b =  0.5; wm2a =  3./20;
  wm1f = -1  ; wm1e =  2.5 ; wm1d = 0; wm1c = -2  ; wm1b = -1  ; wm1a =  2./5;
  wm0f =  1  ; wm0e = -2.5 ; wm0d = 0; wm0c =  3  ; wm0b =  0  ; wm0a = -11./10;
  wp1f = -0.5; wp1e =  1.25; wp1d = 0; wp1c = -2  ; wp1b =  1  ; wp1a =  2./5;
  wp2f =  0.1; wp2e = -0.25; wp2d = 0; wp2c =  0.5; wp2b = -0.5; wp2a =  3./20;

  switch (i) {
  case 0: result = ((((wm3f*t +wm3e)*t +wm3d)*t +wm3c)*t +wm3b)*t +wm3a; break;
  case 1: result = ((((wm2f*t +wm2e)*t +wm2d)*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 2: result = ((((wm1f*t +wm1e)*t +wm1d)*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 3: result = ((((wm0f*t +wm0e)*t +wm0d)*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 4: result = ((((wp1f*t +wp1e)*t +wp1d)*t +wp1c)*t +wp1b)*t +wp1a; break;
  case 5: result = ((((wp2f*t +wp2e)*t +wp2d)*t +wp2c)*t +wp2b)*t +wp2a; break;
  default: result = 0;
  }
  return result;
}

float d2_c3_3ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm3h =   14./3; wm3g = - 49./3; wm3f =   39./2; wm3e = - 95./12; wm3d = 0; wm3c =  0  ; wm3b =  0  ; wm3a =  0;
  wm2h = - 70./3; wm2g =  245./3; wm2f = -195./2; wm2e =  475./12; wm2d = 0; wm2c =  0.5; wm2b =  0.5; wm2a = -1./12;
  wm1h =  140./3; wm1g = -490./3; wm1f =  195   ; wm1e = -475./6 ; wm1d = 0; wm1c = -2  ; wm1b = -1  ; wm1a =  4./3;
  wm0h = -140./3; wm0g =  490./3; wm0f = -195   ; wm0e =  475./6 ; wm0d = 0; wm0c =  3  ; wm0b =  0  ; wm0a = -5./2;
  wp1h =   70./3; wp1g = -245./3; wp1f =  195./2; wp1e = -475./12; wp1d = 0; wp1c = -2  ; wp1b =  1  ; wp1a =  4./3;
  wp2h = - 14./3; wp2g =   49./3; wp2f = - 39./2; wp2e =   95./12; wp2d = 0; wp2c =  0.5; wp2b = -0.5; wp2a = -1./12;

  switch (i) {
  case 0: result = ((((((wm3h*t +wm3g)*t +wm3f)*t +wm3e)*t +wm3d)*t +wm3c)*t +wm3b)*t +wm3a; break;
  case 1: result = ((((((wm2h*t +wm2g)*t +wm2f)*t +wm2e)*t +wm2d)*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 2: result = ((((((wm1h*t +wm1g)*t +wm1f)*t +wm1e)*t +wm1d)*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 3: result = ((((((wm0h*t +wm0g)*t +wm0f)*t +wm0e)*t +wm0d)*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 4: result = ((((((wp1h*t +wp1g)*t +wp1f)*t +wp1e)*t +wp1d)*t +wp1c)*t +wp1b)*t +wp1a; break;
  case 5: result = ((((((wp2h*t +wp2g)*t +wp2f)*t +wp2e)*t +wp2d)*t +wp2c)*t +wp2b)*t +wp2a; break;
  default: result = 0;
  }
  return result;
}

float d2_c3_4ef(float a, float t) {

  float result;
  int i;

  i = (t<0) ? (int)t-1:(int)t;
  t = t - i;

  wm4f =   1./24; wm4e = - 1./12; wm4d =  0    ; wm4c =  0    ; wm4b =  0    ; wm4a =  0;
  wm3f = - 7./24; wm3e =   5./8 ; wm3d =  1./12; wm3c = -1./12; wm3b = - 1./8; wm3a = - 1./24;
  wm2f =   7./8 ; wm2e = - 2    ; wm2d = -1./3 ; wm2c =  1    ; wm2b =  1    ; wm2a =   1./6;
  wm1f = -35./24; wm1e =  85./24; wm1d =  5./12; wm1c = -13./4; wm1b = -13./8; wm1a =  17./24;
  wm0f =  35./24; wm0e = -15./4 ; wm0d =  0    ; wm0c =  14./3; wm0b =  0    ; wm0a = - 5./3;
  wp1f = - 7./8 ; wp1e =  19./8 ; wp1d = -5./12; wp1c = -13./4; wp1b =  13./8; wp1a =  17./24;
  wp2f =   7./24; wp2e = - 5./6 ; wp2d =  1./3 ; wp2c =  1    ; wp2b = -1    ; wp2a =  1./6;
  wp3f = - 1./24; wp3e =   1./8 ; wp3d = -1./12; wp3c = -1./12; wp3b =   1./8; wp3a = -1./24;

  switch (i) {
  case 0: result = ((((wm4f*t +wm4e)*t +wm4d)*t +wm4c)*t +wm4b)*t +wm4a; break;
  case 1: result = ((((wm3f*t +wm3e)*t +wm3d)*t +wm3c)*t +wm3b)*t +wm3a; break;
  case 2: result = ((((wm2f*t +wm2e)*t +wm2d)*t +wm2c)*t +wm2b)*t +wm2a; break;
  case 3: result = ((((wm1f*t +wm1e)*t +wm1d)*t +wm1c)*t +wm1b)*t +wm1a; break;
  case 4: result = ((((wm0f*t +wm0e)*t +wm0d)*t +wm0c)*t +wm0b)*t +wm0a; break;
  case 5: result = ((((wp1f*t +wp1e)*t +wp1d)*t +wp1c)*t +wp1b)*t +wp1a; break;
  case 6: result = ((((wp2f*t +wp2e)*t +wp2d)*t +wp2c)*t +wp2b)*t +wp2a; break;
  case 7: result = ((((wp3f*t +wp3e)*t +wp3d)*t +wp3c)*t +wp3b)*t +wp3a; break;
  default: result = 0;
  }
  return result;
}
